@@ -1,9 +1,9 @@
 import vector_utils
 import numpy
 import svg.path
-import xml.etree.ElementTree as ET
-import os
+import lxml.etree as ET
 import copy
+import warnings
 
 '''
 Compress svg figure through a variety of methods
@@ -32,10 +32,18 @@ class Compress:
         '''
         self.filename, self.extension = svg_file.split('.') # Get filename and extension
         assert self.extension == 'svg', 'File must be an svg'
-        self.figure_data = ET.parse(svg_file) # Parse svg data as xml
+        try:
+            self.figure_data = ET.parse(svg_file) # Parse svg data as xml
+        except ET.XMLSyntaxError, e:
+            '''
+            Large svgs may trigger lxml's 'excessive depth in document' error
+            '''
+            warnings.warn('lxml error: %s - Trying huge xml parser' %(e.message))
+            huge_parser = ET.XMLParser(huge_tree = True)
+            self.figure_data = ET.parse(svg_file, parser = huge_parser)
         self.root = self.figure_data.getroot() # Root object in svg
         self.reportString = '----------- REPORT -----------' # String used to generate reports
-        self.initial_kb = self._get_kb(svg_file) # Get the uncompressed file size in KBytes
+        self.initial_kb = vector_utils.get_kb(svg_file) # Get the uncompressed file size in KBytes
     
     def find_paths(self):
         '''
@@ -271,7 +279,7 @@ class Compress:
             
     def write(self, outputfile = None):
         '''
-        Write compressed svg to file
+        Write compressed svg to file. Not necessary if self.optimize() has been used.
         Inputs:
             outputfile - filename to write to (e.g. 'output.svg'). If not given,
                 will write to 'originalfilename_compressed.svg'
@@ -279,9 +287,10 @@ class Compress:
         if outputfile == None:
             outputfile = '%s_compressed.svg' %(self.filename)
         self.figure_data.write(outputfile)
-        final_kb = self._get_kb(outputfile) # Get the compressed size in KBytes
+        final_kb = vector_utils.get_kb(outputfile) # Get the compressed size in KBytes
         self._append_report('Compressed %sKB to %sKB' %(self.initial_kb, final_kb))
         self._append_report('Saved to %s' %(outputfile))
+        return outputfile
     
     def report(self):
         '''
@@ -301,37 +310,60 @@ class Compress:
         e.g. if the root tag is '{http://www.w3.org/2000/svg}svg', all other tags 
         in the xml are probably prefixed by '{http://www.w3.org/2000/svg}'
         '''
+        # TODO: Fix this hack
         prefix_endindex = self.root.tag.rindex('}')
         return self.root.tag[:(prefix_endindex + 1)]
     
-    def _get_kb(self, filename):
+    def optimize(self, inputfile, outputfile = None):
         '''
-        Return size of the file 'filename' in KB.
+        Use package 'Scour' to optimize the compressed svg (e.g. remove
+        unused nodes).
         '''
-        return os.path.getsize(filename)/1000.0
+        if outputfile == None:
+            outputfile = '%s_opt.svg' %(self.filename)
+#            Defaults:
+#                 {'strip_ids': False, 'shorten_ids': False, 'simple_colors': True, 
+#                  'strip_comments': False, 'remove_metadata': False, 'outfilename': None, 
+#                  'group_create': False, 'protect_ids_noninkscape': False, 'indent_type': 'space', 
+#                  'keep_editor_data': False, 'shorten_ids_prefix': '', 'renderer_workaround': True, 
+#                  'style_to_xml': True, 'protect_ids_prefix': None, 'enable_viewboxing': False, 
+#                  'digits': 5, 'embed_rasters': True, 'infilename': None, 'strip_xml_prolog': False, 
+#                  'group_collapse': True, 'quiet': False, 'protect_ids_list': None}
+#         wrap_scour.optimize(inputfile, outputfile, strip_ids = True, shorten_ids = True, 
+#                             simple_colors = True, strip_comments = True, remove_metadata = True, 
+#                             group_create = True, protect_ids_noninkscape = False,
+#                             indent_type = None, keep_editor_data = False, renderer_workaround = False, 
+#                             style_to_xml = True, digits = 5, embed_rasters = True, strip_xml_prolog = True, 
+#                             group_collapse = True)
+        vector_utils.optimize_svg(inputfile, outputfile)
+        final_kb = vector_utils.get_kb(outputfile) # Get the compressed size in KBytes
+        self._append_report('Compressed %sKB to %sKB' %(self.initial_kb, final_kb))
+        self._append_report('Saved to %s' %(outputfile))
 
 def compress_by_method(filename, compression_type, curve_fidelity = 10, outputfile = None,
-                       pre_select = False, selection_tuple = ('', ''), **kwargs):
+                       pre_select = False, selection_tuple = ('', ''), optimize = True, **kwargs):
     '''
     Convenience function for 1-step compression of svg files.
     Inputs:
-        filename(str) - Path to svg file to compress, e.g. 'test_vector.svg'
+        filename(str) - path to svg file to compress, e.g. 'test_vector.svg'
         compression_type(str) - how to carry out the compression. Options are 'delete' 
           (remove polygons by size), 'simplify' (simplify polygons), and 'merge' (merge 
           neighboring or overlapping polygons).
-        curve_fidelity(int) - All polygons affected by compression (e.g. those that are 
+        curve_fidelity(int) - all polygons affected by compression (e.g. those that are 
           simplified) need to first be linearized so as to convert smooth curves into a set
           of discrete coordinates. curve_fidelity sets the number of coordinates to use to 
           interpolate a curve. The larger the number, the more computation the code will need 
           to perform. Optional, defaults to 10.
-        outputfile(str) - Path to output svg. Optional, defaults to 'originalname_compressed.svg'
-        pre_select(bool) - If True, will perform the compression only on a subset of the 
+        outputfile(str) - path to output svg. Optional, defaults to 'originalname_compressed.svg'
+        pre_select(bool) - if True, will perform the compression only on a subset of the 
           polygons in the svg that are below a certain size threshold. Optional, defaults to False.
-        selection_tuple(tuple) - If pre_select == True, the selection tuple ('criteria', 
+        selection_tuple(tuple) - if pre_select == True, the selection tuple ('criteria', 
           threshold) determines the criteria for selection ('bboxarea' for the area of the 
           bounding box of a polygon, or 'circumference' for its circumference) and the threshold 
           size. Optional, defaults to ('', '')
-        **kwargs - Depending on the compression type and the pre-selection criteria, additional 
+        optimize(bool) - if True, will use the package Scour to optimize compressed svg (e.g.
+          remove redundant nodes, id attributes). Saves optimized svg to 'outputfile_opt.svg'
+        **kwargs - depending on the compression type and the pre-selection criteria, additional 
           key word arguments may be required. See compress_by_merging, compress_by_deletion, and 
           compress_by_simplification in class Compress for the valid keyword arguments when 
           compression_type = 'merge', 'delete', and 'simplify' respectively.
@@ -342,12 +374,6 @@ def compress_by_method(filename, compression_type, curve_fidelity = 10, outputfi
           the success of the compression, the path of the outputted svg, and the initial and 
           compressed file sizes.
     '''
-    def compression_options(compress_instance, option_key, **kwargs):
-        compression_options = {'merge': compress_instance.compress_by_merging,
-                               'delete': compress_instance.compress_by_deletion,
-                               'simplify': compress_instance.compress_by_simplification}
-        compression_options[option_key](**kwargs)
-
     test_compression = Compress(filename)
     test_compression.find_paths()
     test_compression.linearize_paths(curve_fidelity = curve_fidelity)
@@ -355,8 +381,17 @@ def compress_by_method(filename, compression_type, curve_fidelity = 10, outputfi
     if pre_select:
         criteria, threshold = selection_tuple
         pre_selection = test_compression.search_polygons(criteria, threshold)
-    compression_options(test_compression, compression_type, search_within = pre_selection, **kwargs)
-    test_compression.write(outputfile = outputfile)
+
+    compression_options = {'merge': test_compression.compress_by_merging,
+                           'delete': test_compression.compress_by_deletion,
+                           'simplify': test_compression.compress_by_simplification}
+    compress_by_x = compression_options[compression_type]
+    compress_by_x(search_within = pre_selection, **kwargs)
+    inputfile = test_compression.write(outputfile)
+    if optimize:
+        outfilename, extension = outputfile.split('.') # Get filename and extension
+        opti_output = '%s_opt.%s' %(outfilename, extension)
+        test_compression.optimize(inputfile, opti_output)
     test_compression.report()
 
 def test(filename, thresholds = (5, 2), epsilon = 0.5, curve_fidelity = 10, 
@@ -387,12 +422,13 @@ def test(filename, thresholds = (5, 2), epsilon = 0.5, curve_fidelity = 10,
                        selection_tuple = (criterias[0], thresholds[0]), epsilon = epsilon, 
                        bufferDistance = buffer_distance, operation_key = 'hull')
     
-    ## Lastly, see if uncompressed file is same as original:
-    # TODO: Find out why wierdly, it never is.
-    test_compression = Compress(filename)
-    outputfile = ''.join((filestr, '_uncompressed.', extension))
-    test_compression.write(outputfile = outputfile)
-    test_compression.report()
+    ## Lastly, try straight optimization of original file:
+    outputfile = ''.join((filestr, '_opt.', extension))
+    vector_utils.optimize_svg(filename, outputfile)
+    print '----------- REPORT -----------'
+    print 'Optimized %s to %s' %(vector_utils.get_kb(filename), vector_utils.get_kb(outputfile))
+    print 'Saved to %s' %(outputfile)
+    
 
 if __name__ == '__main__':
     # Demonstrate all functions with pre-prepared test vector
