@@ -3,9 +3,13 @@ import shapely.geometry
 import shapely.ops
 import warnings
 import rdp
+import wrap_scour
+import os
 
 '''
 Methods for working with path and style data from svg files
+Written July 22, 2014
+Author: Gen Del Raye
 '''
 
 class nullpoly():
@@ -86,11 +90,20 @@ class Polygon:
         self.poly = self.poly.union(other.poly)
         
     def cascading_union(self, other_list):
+        '''
+        Return union of self.poly and multiple polygons in other_list
+        Shapely.ops.cascaded_union is preferred over poly.union because
+        the former is more efficient for calculating the union of multiple
+        polygons at once.
+        '''
         poly_list = [o.poly for o in other_list]
         poly_list.append(self.poly)
         self.poly = shapely.ops.cascaded_union(poly_list)
         
     def coords(self):
+        '''
+        Return the coordinates of the current polygon
+        '''
         try:
             return list(self.poly.exterior.coords)
         except AttributeError, e:
@@ -100,9 +113,15 @@ class Polygon:
                 return list(self.poly[0].exterior.coords)
         
     def buffer(self, distance):
+        '''
+        Expand the polygon by adding a 'buffer'
+        '''
         self.poly = self.poly.buffer(distance)
     
     def hull(self, other_list):
+        '''
+        Calculate the minimum convex hull of self.poly and the polygons in other_list
+        '''
         poly_coords = self.coords()
         for o in other_list:
             poly_coords.extend(o.coords())
@@ -212,4 +231,21 @@ def parse_style(stylestr):
     '''
     stylestr = stylestr.split(';') # Split by entry
     styledict = dict([(s.split(':')[0], s.split(':')[-1]) for s in stylestr])
-    return styledict
\ No newline at end of file
+    return styledict
+
+def optimize_svg(filename, outputname):
+    '''
+    Use package Scour to optimize an svg rather than compress it
+    '''
+    wrap_scour.optimize(filename, outputname, strip_ids = True, shorten_ids = True, 
+                        simple_colors = True, strip_comments = True, remove_metadata = True, 
+                        group_create = True, protect_ids_noninkscape = False,
+                        indent_type = None, keep_editor_data = False, renderer_workaround = False, 
+                        style_to_xml = True, digits = 5, embed_rasters = True, strip_xml_prolog = True, 
+                        group_collapse = True)
+
+def get_kb(filename):
+    '''
+    Return size of the file 'filename' in KB.
+    '''
+    return os.path.getsize(filename)/1000.0
\ No newline at end of file
